@@ -1,4 +1,5 @@
-from typing import Dict, List
+from typing import Dict, List, Union
+from io import StringIO, TextIOWrapper, BytesIO
 import requests
 import os
 import logging
@@ -8,13 +9,29 @@ class DuetAPI:
     api_name = ''
 
     def __init__(self, base_url: str) -> None:
-        if not base_url.startswith('http://'):
-            base_url = f'http://{base_url}'
-        self._base_url = base_url
+        self.base_url = base_url
 
+    def __repr__(self):
+        return f"DuetWebAPI('{self.base_url}') - {self.api_name}"
+
+    @property
     def base_url(self):
         return self._base_url
 
+    @base_url.setter
+    def base_url(self, value: str):
+        if not value.startswith('http://'):
+            value = f'http://{value}'
+        self._base_url = value
+
+    def connect(self, **kwargs):
+        """ Start connection to Duet """
+        raise NotImplementedError
+
+    def disconnect(self):
+        """ End connection to Duet """
+        raise NotImplementedError
+
     def get_model(self, key: str = None) -> Dict:
         """ Get Duet object model. RRF3 only """
         raise NotImplementedError
@@ -23,11 +40,11 @@ class DuetAPI:
         """ Send G/M/T-code to Duet """
         raise NotImplementedError
 
-    def get_file(self, filename: str, directory: str = 'gcodes') -> str:
+    def get_file(self, filename: str, directory: str = 'gcodes', binary: bool = False) -> str:
         """ Get file from Duet """
         raise NotImplementedError
 
-    def upload_file(self, file: str, directory: str = 'gcodes') -> Dict:
+    def upload_file(self, file: Union[str, bytes, StringIO, TextIOWrapper, BytesIO], filename: str, directory: str = 'gcodes') -> Dict:
         """ Upload file to Duet """
         raise NotImplementedError
 
@@ -50,5 +67,3 @@ class DuetAPI:
     def create_directory(self, directory: str) -> Dict:
         """ Create a new directory """
         raise NotImplementedError
-
-pass
\ No newline at end of file
